# -*- coding: utf-8 -*-

import pymysql
from DBUtils.PooledDB import PooledDB


class MysqlConnectionPool():
    __pool = None

    def __init__(self,
                 db, DB_MIN_CACHED=None, DB_MAX_CACHED=None, DB_MAX_SHARED=None, DB_MAX_CONNECTIONS=None,
                 DB_MAX_USAGE=None, MYSQL_DB_HOST=None, MYSQL_DB_USER=None, MYSQL_DB_PWD=None, MYSQL_PORT=None,
                 ):
        self.db = db
        self.DB_MIN_CACHED = 5 if DB_MIN_CACHED is None else DB_MIN_CACHED
        self.DB_MAX_CACHED = 10 if DB_MAX_CACHED is None else DB_MAX_CACHED
        self.DB_MAX_SHARED = 30 if DB_MAX_SHARED is None else DB_MAX_SHARED
        self.DB_MAX_CONNECTIONS = 50 if DB_MAX_CONNECTIONS is None else DB_MAX_CONNECTIONS
        self.DB_MAX_USAGE = 0 if DB_MAX_USAGE is None else DB_MAX_USAGE
        self.MYSQL_DB_HOST = MYSQL_DB_HOST
        self.MYSQL_DB_USER = MYSQL_DB_USER
        self.MYSQL_DB_PWD = MYSQL_DB_PWD
        self.MYSQL_PORT = 3306 if MYSQL_PORT is None else MYSQL_PORT

        """
        DB_MIN_CACHED:最少的空闲连接数，如果空闲连接数小于这个数，pool会创建一个新的连接
        DB_MAX_CACHED:最大的空闲连接数，如果空闲连接数大于这个数，pool会关闭空闲连接
        DB_MAX_SHARED:当连接数达到这个数，新请求的连接会分享已经分配出去的连接
        DB_MAX_CONNECTIONS:最大的连接数
        maxusage: 单个连接的最大允许复用次数(缺省值 0 或 False 代表不限制的复用).当达到最大数时,连接会自动重新连接(关闭和重新打开)
        DB_MAX_USAGE: 
        """

    # 创建数据库连接conn和游标cursor
    def __enter__(self):
        self.conn = self.__getconn()
        self.cursor = self.conn.cursor()

    def __getconn(self):
        if self.__pool is None:
            self.__pool = PooledDB(
                creator=pymysql,
                mincached=self.DB_MIN_CACHED,
                maxcached=self.DB_MAX_CACHED,
                maxshared=self.DB_MAX_SHARED,
                maxconnections=self.DB_MAX_CONNECTIONS,
                blocking=True,
                maxusage=self.DB_MAX_USAGE,
                host=self.MYSQL_DB_HOST,
                user=self.MYSQL_DB_USER,
                passwd=self.MYSQL_DB_PWD,
                db=self.db,
                port=self.MYSQL_PORT,
                cursorclass=pymysql.cursors.DictCursor
            )

        return self.__pool.connection()

    # 释放连接池资源
    def __exit__(self, exc_type, exc_val, exc_tb):
        self.cursor.close()
        self.conn.close()

    def __del__(self):
        if self.__pool:
            self.__pool.close()

    # 关闭连接归还给链接池
    def close(self):
        self.cursor.close()
        self.conn.close()

    # 从连接池中取出一个连接
    def getconn(self):
        conn = self.__getconn()
        cursor = conn.cursor()
        return cursor, conn



# -*- coding: utf-8 -*-
# @Author: vekair

import logging
from mysql_pool import MysqlConnectionPool
log = logging.Logger(__name__)

class MysqlExecute():
    def __init__(self, db):
        self.db = MysqlConnectionPool(db)

    # 封装执行命令
    def execute(self, sql, param=None, autoclose=False):
        """
        【主要判断是否有参数和是否执行完就释放连接】
        :param sql: 字符串类型，sql语句
        :param param: sql语句中要替换的参数"select %s from tab where id=%s" 其中的%s就是参数
        :param autoclose: 是否关闭连接
        :return: 返回连接conn和游标cursor
        """
        cursor, conn = self.db.getconn()  # 从连接池获取连接

        count = 0
        try:
            # count : 为改变的数据条数
            if param:
                count = cursor.execute(sql, param)
            else:
                count = cursor.execute(sql)
            conn.commit()
            if autoclose:
                self.close(cursor, conn)
        except Exception as e:
            log.warn(e)
        return cursor, conn, count

    def close(self, cursor, conn):
        """释放连接归还给连接池"""
        cursor.close()
        conn.close()

    # 增加
    def insertone(self, sql, param):
        conn = cursor = None
        try:
            cursor, conn, count = self.execute(sql, param)
            _id = cursor.lastrowid  # 获取当前插入数据的主键id，该id应该为自动生成为好
            conn.commit()
            self.close(cursor, conn)
            # return count
            # 防止表中没有id返回0
            if _id == None or _id == 0:
                return True
            return _id
        except Exception as e:
            log.warn(e)
            if conn:
                conn.rollback()
                self.close(cursor, conn)

    # 查询所有
    def selectall(self, sql, param=None):
        cursor = conn = None
        try:
            cursor, conn, count = self.execute(sql, param)
            res = cursor.fetchall()
            return res
        except Exception as e:
            log.warn(e)
            if conn:
                self.close(cursor, conn)

    # 查询单条
    def selectone(self, sql, param=None):
        cursor = conn = None
        try:
            cursor, conn, count = self.execute(sql, param)
            res = cursor.fetchone()
            self.close(cursor, conn)
            return res
        except Exception as e:
            log.warn("error_msg:", e.args)
            if conn:
                self.close(cursor, conn)

    # 更新
    def update(self, sql, param=None):
        conn = cursor = None
        try:
            cursor, conn, count = self.execute(sql, param)
            conn.commit()
            self.close(cursor, conn)
            return count
        except Exception as e:
            log.warn(e)
            if conn:
                conn.rollback()
                self.close(cursor, conn)



# -*- coding: utf-8 -*-
# @Author: vekair

import redis


class RedisPool(object):
    def __init__(self, REDIS_HOST, REDIS_PORT, REDIS_PASS):
        self.host = REDIS_HOST
        self.port = REDIS_PORT
        self.pass_word = REDIS_PASS

    def build_redis(self, redis_db=0):
        """建立链接"""
        rdp = redis.ConnectionPool(host=self.host, port=self.port, db=redis_db, password=self.pass_word)
        return redis.StrictRedis(connection_pool=rdp)
